//S88 control board

// reference : RB_S88
// Software by Ruud Boer, November 2014.
// https://rudysmodelrailway.wordpress.com/software/

//151101 fujigaya2  6ch photo detector version 
//151114 fujigaya2 (111507 board debug)
//151121 fujigaya2 when one of the sensors is on  ,RUN LED is on.
//151230 fujigaya2 change original source
//180316 8ch ver for Arduino nano
//180512 SpeedMeter

//OLED
#include "OzOLED.h"

/* LCD functions */
OzOLED LCD;

//int clk_counter=0;
//const byte dataIn=0;  //data input from next Arduino in S88 chain
//const byte dataOut=1; //data output pin=6

//Sensor control
#define RUN_LED_PIN 13 // RUN_LED(status LED) on/off pin
#define SENSOR_INTERVAL 10 //sensor interval[ms]
#define ANALOG_PORT_NUM 2 //analog port number 
#define CONT_PORT_NUM 16 //port_num to see from DCC Controller

//Pin 関係のdefine
//IR LED output port
#define IR_LED_PIN0  12 // IR_LED(sensor LED) on/off pin
#define IR_LED_PIN1  11 // IR_LED(sensor LED) on/off pin
//Analog input port
#define ANALOG_PIN0  0 // analog pin number
#define ANALOG_PIN1  1 // analog pin number

//SpeedMeter定数
// Kato HO
// #define SENSOR_DISTANCE 552 //センサ間の距離 mm
// #define SCALE 87 //スケール  1/80なら80
#define SENSOR_DISTANCE (280 * 2 + 70) //センサ間の距離 mm
#define SCALE 150 //スケール  1/80なら80
#define SENSOR_IGNORED_TIME 1000 //センサ不感時間


int led_pin[ANALOG_PORT_NUM] = {
  IR_LED_PIN0,
  IR_LED_PIN1,
  };
  
int analog_pin[ANALOG_PORT_NUM] = {
  ANALOG_PIN0,
  ANALOG_PIN1,
  };

unsigned long gPreviousL5 = 0; 
boolean ir_led_onoff =false;
int reflector_state = 0;
int light_on_value[ANALOG_PORT_NUM];
int light_off_value[ANALOG_PORT_NUM];
int diff_value[ANALOG_PORT_NUM];

int sensor_value[CONT_PORT_NUM]; 

int diff_value_threshold = 50;

enum measure_state
{
  wait_start,
  forward_start,reverse_start,
  wait_stop
};
measure_state m_state = wait_start;
unsigned long time_count = 0; 
float loco_speed = 0;//km/h
unsigned long gPreviousL6 = 0; 

void setup() {
  /* OLED init */
  LCD.init();
  LCD.clearDisplay();           //clear the screen and set start position to top left corner
  LCD.setNormalDisplay();       //Set display to Normal mode
  LCD.setPageMode();            //Set addressing mode to Page Mode

    
  //S88 Control
  pinMode(2, INPUT_PULLUP);
  pinMode(3, INPUT_PULLUP);
  
  //Sensor Control
  for(int i = 0;i < ANALOG_PORT_NUM;i++)
  {
    pinMode(led_pin[i], OUTPUT);
  }
  //Status LED Control
  pinMode(RUN_LED_PIN, OUTPUT);
  gPreviousL5 = millis();
  reflector_state = 0;
  //Serial.begin(115200);
  
  //init sensor_value,transfer_value
  for(int i = 0; i < CONT_PORT_NUM;i++)
  {
    sensor_value[i] = 0;
  }

  LCD.lcd_clear();
  LCD.printString("SpeedMeter Test", 0, 0);
  LCD.printString("18/05/14 K1", 0, 1);
  char str_temp[10];
  sprintf(str_temp,"Distance:%4dmm",SENSOR_DISTANCE);
  LCD.printString(str_temp, 0, 2);
  sprintf(str_temp,"Scale 1/%3d",SCALE);
  LCD.printString(str_temp, 0, 3);
  
  delay(5000) ;
  LCD.lcd_clear();
  LCD.printString("wait_start     ",0,4);
  
  
}

void loop()
{
  if(millis() - gPreviousL5 > SENSOR_INTERVAL)
  {
    gPreviousL5 = millis();
    detect_analog();
  }

  byte current_sensor_value = sensor_value[0] || (sensor_value[1]<<1);
  switch(m_state)
  {
    case wait_start:
      //値が変わったら 計測開始
      if(sensor_value[0] == 1)
      {
        m_state = forward_start;
        time_count = millis();
        LCD.printString("forward_start  ",0,4);
      }
      else if (sensor_value[1] == 1)
      {
        m_state = reverse_start;
        time_count = millis();
        LCD.printString("receive_start  ",0,4);
      }
      reset_sensor_value();
      break;
    case forward_start:
      //もう一方のセンサが反応したら計測終了
      if(sensor_value[1] == 1)
      {
        m_state = wait_stop;
        time_count = millis() - time_count;
        float loco_speed = (float)SENSOR_DISTANCE / (float)time_count * SCALE; // 実縮尺でmm/ms → m/s
        loco_speed *= 3600 / 1000;
        char str_speed[10];
        sprintf(str_speed,"%4d",(long)loco_speed);
        LCD.printBigNumberX2(str_speed, 0, 0);
        LCD.printString("km/h", 10, 3);
        LCD.printString("ignore  sensor ",0,4);
        gPreviousL6 = millis();
      }
      reset_sensor_value();
      break;
    case reverse_start:
      //もう一方のセンサが反応したら計測終了
      if(sensor_value[0] == 1)
      {
        m_state = wait_stop;
        time_count = millis() - time_count;
        float loco_speed = (float)SENSOR_DISTANCE / (float)time_count * SCALE; // 実縮尺でmm/ms → m/s
        loco_speed *= 3600 / 1000;
        char str_speed[10];
        sprintf(str_speed,"%4d",(long)loco_speed);
        LCD.printBigNumberX2(str_speed, 0, 0);
        LCD.printString("km/h", 10, 3);
        LCD.printString("ignore  sensor ",0,4);
        gPreviousL6 = millis();
      }
      reset_sensor_value();
      break;
    case wait_stop:
      if( millis() - gPreviousL6 > SENSOR_IGNORED_TIME)
      {
        //SENSOR_IGNORED_TIMEごとに 電車が通り過ぎたかどうかを確認
        if((sensor_value[0] == 0) && (sensor_value[1] == 0))
        {
          m_state = wait_start;
          LCD.printString("wait_start     ",0,4);
        }
        gPreviousL6 = millis();
        reset_sensor_value();
      }
      break;
    default:
      LCD.printString("error_state    ",0,4);
      break;
  }
}      

void reset_sensor_value()
{
    //reset sensor value
  for(int i = 0; i < ANALOG_PORT_NUM;i++)
  {
    sensor_value[i] = 0;
  }
}

void detect_analog()
{
  //LEDは個別なので、ルーチンは変える
  int sensor_state = 0;
  for(int i= 0;i < ANALOG_PORT_NUM;i++)
  {
      //直列に書く
      //LED on
      IR_LED_onoff(i,1);
      //wait
      delayMicroseconds(2000) ;
      //Read Photo Transistor value at LED ON  
      light_on_value[i] = analogRead(i);
      //LED off
      IR_LED_onoff(i,0);
      //wait
      delayMicroseconds(2000) ;
      //Read Photo Transistor value atLED Off  
      light_off_value[i] = analogRead(i);
      diff_value[i] = abs(light_on_value[i] - light_off_value[i]);
      if(diff_value[i] > diff_value_threshold)
      {
        sensor_value[i] = 1;
        sensor_state ++;
      }
  }
  
  //RUN_LEDのところの話
  if(sensor_state != 0)
  {
      digitalWrite(RUN_LED_PIN,HIGH);
  }
  else
  {
      digitalWrite(RUN_LED_PIN,LOW);
  }
}

void IR_LED_onoff(int ch,int on_off)
{
    //1:on 0:off
    digitalWrite(led_pin[ch],on_off);
}

